// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "styles.h"
#include "excitwin.h"


Excitwin::Excitwin (X_window *parent, int xp, int yp, Vocoder *vocoder, int ident) :
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _vocoder (vocoder),
    _ident (ident)
{
    _bintl = new Pbutt1 (this, this, B_INTL, bintl_img, 8, 15, 30, 18);
    _bintl->x_map ();
    
    _gain_geom._backg = XftColors [C_MAIN_BG];
    _gain_geom._image [0] = excitctl_img;
    _gain_geom._lncol [0] = 0;
    _gain_geom._x0 = 6;
    _gain_geom._y0 = 5;
    _gain_geom._dx = 39;
    _gain_geom._dy = 39;
    _gain_geom._xref = 19.5;
    _gain_geom._yref = 19.5;
    _gain_geom._rad = 13;

    _eq1g_geom._backg = XftColors [C_MAIN_BG];
    _eq1g_geom._image [0] = excitctl_img;
    _eq1g_geom._lncol [0] = 1;
    _eq1g_geom._x0 = 63;
    _eq1g_geom._y0 = 7;
    _eq1g_geom._dx = 35;
    _eq1g_geom._dy = 35;
    _eq1g_geom._xref = 17.5;
    _eq1g_geom._yref = 17.5;
    _eq1g_geom._rad = 11;

    _eq2g_geom._backg = XftColors [C_MAIN_BG];
    _eq2g_geom._image [0] = excitctl_img;
    _eq2g_geom._lncol [0] = 1;
    _eq2g_geom._x0 = 113;
    _eq2g_geom._y0 = 7;
    _eq2g_geom._dx = 35;
    _eq2g_geom._dy = 35;
    _eq2g_geom._xref = 17.5;
    _eq2g_geom._yref = 17.5;
    _eq2g_geom._rad = 11;

    _rgain = new Rgainctl (this, this, R_GAIN, &_gain_geom, 49, 0, 30);
    _rgain->set_value (0.0f);
    _rgain->x_map ();
    _req1g = new Rlinctl (this, this, R_EQ1G, &_eq1g_geom, 49, 0, 72, 2, -18.0f, 18.0f, 0.0f);
    _req1g->x_map ();
    _req2g = new Rlinctl (this, this, R_EQ2G, &_eq2g_geom, 49, 0, 72, 2, -18.0f, 18.0f, 0.0f);
    _req2g->x_map ();
    
    if (_ident == 'V')
    {
	_vocoder->set_Vgain (_rgain->value ());
	_vocoder->set_Veq1g (0.0f);
	_vocoder->set_Veq2g (0.0f);
    }
    if (_ident == 'U')
    {
	_vocoder->set_Ugain (_rgain->value ());
	_vocoder->set_Ueq1g (0.0f);
	_vocoder->set_Ueq2g (0.0f);
    }

    x_add_events (ExposureMask); 
    x_map ();
}

 
Excitwin::~Excitwin (void)
{
}

 
void Excitwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;
    }
}


void Excitwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Excitwin::redraw (void)
{
    XPutImage (dpy (), win (), dgc (), excitctl_img, 0, 0, 49, 0, 170, 57);
}


void Excitwin::handle_callb (int type, X_window *W, XEvent *E)
{
    RotaryCtl  *R;
    PushButton *B;
    
    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
        switch (R->cbind ())
	{
	case R_GAIN:    
	    if (_ident == 'V') _vocoder->set_Vgain (R->value ());
	    if (_ident == 'U') _vocoder->set_Ugain (R->value ());
	    break;
	case R_EQ1G:    
	    if (_ident == 'V') _vocoder->set_Veq1g (R->value ());
	    if (_ident == 'U') _vocoder->set_Ueq1g (R->value ());
	    break;
	case R_EQ2G:    
	    if (_ident == 'V') _vocoder->set_Veq2g (R->value ());
	    if (_ident == 'U') _vocoder->set_Ueq2g (R->value ());
	    break;
	}
	break;
    case PushButton::PRESS:
	B = (PushButton *) W;
	switch (B->cbind ())
	{
	case B_INTL:
	    if (_ident == 'V') _vocoder->set_Vintern (B->state () & 2);
	    if (_ident == 'U') _vocoder->set_Uintern (B->state () & 2);
	    break;
	}
	break;
    }
}


