// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#ifndef __FILTBANK3_H
#define __FILTBANK3_H


class VocFilter
{
public:

    VocFilter (void) {};
    ~VocFilter (void) {};

    void init (const float *c);
    void reset (void);
    void process (int n, float *inp, float *out);

private:

    const float *_c;
    float        _z [6];
};


class VocFiltbank
{
public:

    enum { NFILT = 24 };
    
    VocFiltbank (void) {};
    ~VocFiltbank (void) {};

    void init (int fsam);
    void reset (void);
    VocFilter *filter (int i) { return _filt + i; }
    
protected:

    VocFilter _filt [NFILT];
};


class VocAnalyser : public VocFiltbank
{
public:

    VocAnalyser (int frag, int fsam);
    ~VocAnalyser (void);

    void  reset (void);
    void  process (float *inp);
    float power (int i) { return _p2 [i]; }
    float ratio (void) { return _rp; }
    
private:

    int          _frag;
    int          _fsam;
    float        _p1 [NFILT];
    float        _p2 [NFILT];
    float        _w [NFILT];
    float        _rp;
    float       *_inpb;
    float       *_outb;
};


class VocResynth : public VocFiltbank
{
public:

    VocResynth (int frag, int fsam);
    ~VocResynth (void);

    void reset (void);
    void process (float *inp, float *out);
    void set_gain (int i, float g) { _g1 [i] = g; }
    
private:

    int          _frag;
    float        _fsam;
    float        _g0 [NFILT];
    float        _g1 [NFILT];
    float       *_outb;
};


#endif
