// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "styles.h"
#include "filtwin.h"


Filtwin::Filtwin (X_window *parent, int xp, int yp, Vocoder *vocoder) :
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _vocoder (vocoder)
{
    int  j, x, y;

    _gain_geom._backg = XftColors [C_MAIN_BG];
    _gain_geom._image [0] = filtgain_img;
    _gain_geom._lncol [0] = 1;
    _gain_geom._x0 = 0;
    _gain_geom._y0 = 0;
    _gain_geom._dx = 29;
    _gain_geom._dy = 29;
    _gain_geom._xref = 14.5;
    _gain_geom._yref = 14.5;
    _gain_geom._rad = 10;
    
    x = 8;
    y = 20;
    for (j = 0; j < NBAND; j++)
    {
        _rbgain [j] = new Rlinctl (this, this, j, &_gain_geom, x, y, 300, 5, -30.0, 30.0, 0.0);
	_rbgain [j]->x_map ();
	_vocoder->set_gband (j, 1.0f);
	_levled [j] = new Mcled (this, x + 9, 5, ledcolor_img);
	x += XSTEP;
	if (j & 1) y -= YSTEP; else y += YSTEP;
    }

    x = 60;
    y = 77;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "204", 0))->x_map ();
    x += 4 * XSTEP;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "440", 0))->x_map ();
    x += 4 * XSTEP;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "880", 0))->x_map ();
    x += 4 * XSTEP;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "1k76", 0))->x_map ();
    x += 4 * XSTEP;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "3k61", 0))->x_map ();
    x += 4 * XSTEP;
    (new X_textln (this, &tstyle2, x, y, 50, 17, "11k7", 0))->x_map ();

    x_add_events (ExposureMask); 
    x_map (); 
}

 
Filtwin::~Filtwin (void)
{
}

 
void Filtwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;
    }
}


void Filtwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Filtwin::redraw (void)
{
    int  i, x;
    X_draw D (dpy (), win (), dgc (), xft ());

    D.setline (1);
    D.setcolor (XftColors [C_MAIN_L1]->pixel);
    for (i = 1; i < NBAND; i += 2)
    {
	x = 22 + i * XSTEP;
	D.drawline (x, 20, x, 45);
    }
    for (i = 0; i < NBAND; i++) _levled [i]->render ();
}


void Filtwin::handle_callb (int type, X_window *W, XEvent *E)
{
    RotaryCtl  *R;
    int         j;

    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
	j = R->cbind ();
	_vocoder->set_gband (j, powf (10.0f, 0.05f * R->value ()));
	break;
    }
}


void Filtwin::update (void)
{
    int    j, k;
    float  p;
    
    for (j = 0; j < NBAND; j++)
    {
	p = _vocoder->get_pband (j, 0.4f);
	if      (p > 2.0f) k = 14;
	else if (p > 0.2f) k = 13;
	else               k = 0;
        _levled [j]->update (k);
    }
}
