// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "styles.h"
#include "inputwin.h"


Inputwin::Inputwin (X_window *parent, int xp, int yp, Vocoder *vocoder) :
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _vocoder (vocoder)
{
    _gain_geom._backg = XftColors [C_MAIN_BG];
    _gain_geom._image [0] = inputctl_img;
    _gain_geom._lncol [0] = 0;
    _gain_geom._x0 = 14;
    _gain_geom._y0 = 4;
    _gain_geom._dx = 39;
    _gain_geom._dy = 39;
    _gain_geom._xref = 19.5;
    _gain_geom._yref = 19.5;
    _gain_geom._rad = 13;

    _uvrt_geom._backg = XftColors [C_MAIN_BG];
    _uvrt_geom._image [0] = inputctl_img;
    _uvrt_geom._lncol [0] = 0;
    _uvrt_geom._x0 = 66;
    _uvrt_geom._y0 = 4;
    _uvrt_geom._dx = 39;
    _uvrt_geom._dy = 39;
    _uvrt_geom._xref = 19.5;
    _uvrt_geom._yref = 19.5;
    _uvrt_geom._rad = 13;

    _rgain = new Rgainctl (this, this, R_GAIN, &_gain_geom, 0, 13, 30);
    _rgain->set_value (0.0f);
    _rgain->x_map ();

    _ruvrt = new Rlinctl (this, this, R_UVRT, &_uvrt_geom, 0, 13, 72, 3, -18, 18, 0);
    _ruvrt->set_value (0.0f);
    _ruvrt->x_map ();

    _Uled = new Mcled (this, 70, 1, ledcolor_img);    
    _Vled = new Mcled (this, 90, 1, ledcolor_img);    
    _Uled->update (3);
    _Vled->update (5);
    
    _bhold = new Pbutt1 (this, this, B_HOLD, bhold_img, 130, 23, 40, 30);
    _bhold->x_map ();
    
    x_add_events (ExposureMask); 
    x_map ();
}

 
Inputwin::~Inputwin (void)
{
}

 
void Inputwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;
    }
}


void Inputwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Inputwin::redraw (void)
{
    XPutImage (dpy (), win (), dgc (), inputctl_img, 0, 0, 0, 13, 119, 59);
    _Uled->render ();
    _Vled->render ();
}

void Inputwin::update (void)
{
    float v = _vocoder->get_vcdf ();
    if      (v >  0.9f) _Vled->update (14);
    else if (v > -0.7f) _Vled->update (13);
    else                _Vled->update (0);
    if      (v < -0.9f) _Uled->update (14);
    else if (v <  0.7f) _Uled->update (13);
    else                _Uled->update (0);
}


void Inputwin::handle_callb (int type, X_window *W, XEvent *E)
{
    RotaryCtl  *R;
    PushButton *B;
    
    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
	switch (R->cbind ())
	{
	case R_GAIN:
	    _vocoder->set_Again (R->value ());
	    break;
	case R_UVRT:
	    _vocoder->set_UVthr (R->value ());
	    break;
	}
	break;
    case PushButton::PRESS:
	B = (PushButton *) W;
	switch (B->cbind ())
	{
	case B_HOLD:
	    _vocoder->set_hold (B->state () & 2);
	    break;
	}
	break;
    }
}


