// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#ifndef __MAINWIN_H
#define	__MAINWIN_H


#include <clxclient.h>
#include "guiclass.h"
#include "jclient.h"
#include "inputwin.h"
#include "oscilwin.h"
#include "excitwin.h"
#include "filtwin.h"
#include "global.h"


class Mainwin : public A_thread, public X_window, public X_callback
{
public:

    enum { XSIZE = 730, YSIZE = 170 };

    Mainwin (X_rootwin *parent, X_resman *xres, int xp, int yp, Jclient *jclient);
    ~Mainwin (void);
    Mainwin (const Mainwin&);
    Mainwin& operator=(const Mainwin&);

    void stop (void) { _stop = true; }
    int process (void); 

private:

    enum { R_MIDI };
    
    virtual void thr_main (void) {}

    void handle_time (void);
    void handle_stop (void);
    void handle_event (XEvent *);
    void handle_callb (int type, X_window *W, XEvent *E);
    void showval (int k);
    void expose (XExposeEvent *E);
    void clmesg (XClientMessageEvent *E);
    void redraw (void);

    Atom            _atom;
    bool            _stop;
    X_resman       *_xres;
    Jclient        *_jclient;
    Vocoder        *_vocoder;
    Oscilwin       *_oscilwin;
    Inputwin       *_inputwin;
    Excitwin       *_excwinV;
    Excitwin       *_excwinU;
    Filtwin        *_filtwin;
    RotaryGeom      _gmidi;
    RotaryCtl      *_rmidi;
    X_tbutton      *_tmidi;
};


#endif
