// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "styles.h"
#include "oscilwin.h"


Oscilwin::Oscilwin (X_window *parent, int xp, int yp, Vocoder *vocoder) :
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _vocoder (vocoder)
{
    _bmidi = new Pbutt1 (this, this, B_MIDI, bmidi_img, 8, 15, 30, 18);
    _bmidi->x_map ();
    
    _freq_geom._backg = XftColors [C_MAIN_BG];
    _freq_geom._image [0] = oscilctl_img;
    _freq_geom._lncol [0] = 0;
    _freq_geom._x0 = 6;
    _freq_geom._y0 = 5;
    _freq_geom._dx = 39;
    _freq_geom._dy = 39;
    _freq_geom._xref = 19.5;
    _freq_geom._yref = 19.5;
    _freq_geom._rad = 13;
    _rfreq = new Rlinctl (this, this, R_FREQ, &_freq_geom, 49, 0, 36, 1, 36, 72, 54);
    _rfreq->x_map ();

    _tune_geom._backg = XftColors [C_MAIN_BG];
    _tune_geom._image [0] = oscilctl_img;
    _tune_geom._lncol [0] = 0;
    _tune_geom._x0 = 61;
    _tune_geom._y0 = 7;
    _tune_geom._dx = 35;
    _tune_geom._dy = 35;
    _tune_geom._xref = 17.5;
    _tune_geom._yref = 17.5;
    _tune_geom._rad = 11;
    _rtune = new Rlinctl (this, this, R_TUNE, &_tune_geom, 49, 0, 100, 2, -2, 2, 0);
    _rtune->x_map ();

    _fmod_geom._backg = XftColors [C_MAIN_BG];
    _fmod_geom._image [0] = oscilctl_img;
    _fmod_geom._lncol [0] = 1;
    _fmod_geom._x0 = 113;
    _fmod_geom._y0 = 7;
    _fmod_geom._dx = 35;
    _fmod_geom._dy = 35;
    _fmod_geom._xref = 17.5;
    _fmod_geom._yref = 17.5;
    _fmod_geom._rad = 11;
    _rfmod = new Rlinctl (this, this, R_FMOD, &_fmod_geom, 49, 0, 100, 2,  0.0f, 1.0f, 0.0f);
    _rfmod->x_map ();

    _lfof_geom._backg = XftColors [C_MAIN_BG];
    _lfof_geom._image [0] = oscilctl_img;
    _lfof_geom._lncol [0] = 1;
    _lfof_geom._x0 = 163;
    _lfof_geom._y0 = 7;
    _lfof_geom._dx = 35;
    _lfof_geom._dy = 35;
    _lfof_geom._xref = 17.5;
    _lfof_geom._yref = 17.5;
    _lfof_geom._rad = 11;
    _rlfof = new Rlogctl (this, this, R_LFOF, &_lfof_geom, 49, 0, 120, 2,  1.0f, 16.0f, 4.0f);
    _rlfof->x_map ();
    
    _vocoder->set_oscfreq (_rfreq->value ());
    _vocoder->set_osctune (_rtune->value ());
    _vocoder->set_oscfmod (_rfmod->value ());
    _vocoder->set_lfofreq (_rlfof->value ());

    x_add_events (ExposureMask); 
    x_map ();
}

 
Oscilwin::~Oscilwin (void)
{
}

 
void Oscilwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;
    }
}


void Oscilwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Oscilwin::redraw (void)
{
    XPutImage (dpy (), win (), dgc (), oscilctl_img, 0, 0, 49, 0, 230, 57);
}


void Oscilwin::handle_callb (int type, X_window *W, XEvent *E)
{
    RotaryCtl  *R;
    PushButton *B;

    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
        switch (R->cbind ())
	{
	case R_FREQ:
	    _vocoder->set_oscfreq (R->value ());
	    break;
	case R_TUNE:    
	    _vocoder->set_osctune (R->value ());
	    break;
	case R_FMOD:    
	    _vocoder->set_oscfmod (R->value ());
	    break;
	case R_LFOF:    
	    _vocoder->set_lfofreq (R->value ());
	    break;
	}
	break;
    case PushButton::PRESS:
	B = (PushButton *) W;
	switch (B->cbind ())
	{
	case B_MIDI:
	    _vocoder->set_midi2osc (B->state () & 2);
	    break;
	}
	break;
    }
}


