// ----------------------------------------------------------------------------
//
//  Copyright (C) 2009-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include <string.h>
#include "pareq.h"


#define PIF 3.141592f
#define P2F 6.283185f
#define EPS 1e-20f


Pareq::Pareq (void) :
    _c1 (0),
    _c2 (0),
    _g (0)
{
    reset ();
}


Pareq::~Pareq (void)
{
}


void Pareq::set_param (float f, float b, float g)
{
    g = powf (10.0f, 0.05f * g); 
    b *= PIF * f;
    b /= sqrtf (g);
    _g = (g - 1) / 2;
    _c1 = -cosf (P2F * f);
    _c2 = (1 - b) / (1 + b);
}


void Pareq::process (int n, float *inp, float *out)
{
    int    i;
    float  x, y, z1, z2;

    z1 = _z1;
    z2 = _z2;
    for (i = 0; i < n; i++)
    {
        x = inp [i];
        y = x - _c2 * z2;
        out [i] = x - _g * (z2 + _c2 * y - x);
        y -= _c1 * z1;
        z2 = z1 + _c1 * y;
        z1 = y + EPS;
    }
   _z1 = z1;
   _z2 = z2;
}

