// ----------------------------------------------------------------------------
//
//  Copyright (C) 2010-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include "styles.h"
#include "png2img.h"


XftColor      *XftColors [NXFTCOLORS];
XftFont       *XftFonts [NXFTFONTS];

X_textln_style tstyle1;
X_textln_style tstyle2;
X_button_style bstyle1;

XImage  *bmidi_img;
XImage  *bintl_img;
XImage  *bhold_img;
XImage  *filtgain_img;
XImage  *ledcolor_img;
XImage  *oscilctl_img;
XImage  *inputctl_img;
XImage  *excitctl_img;
XImage  *midichan_img;


XImage *load_img (const char *file, X_display *disp, const XftColor *color)
{
    char    s [1024];
    XImage  *img;

    sprintf (s, "%s/%s.png", SHARED, file);
    img = png2img (s, disp, color);
    if (img) return img;
    exit (1);
}


void free_img (XImage *image)
{
    image->data = 0;
    XDestroyImage (image);
}


void styles_init (X_display *disp, X_resman *xrm)
{
    XftColors [C_MAIN_BG] = disp->alloc_xftcolor (0.25f, 0.25f, 0.25f, 1.0f);
    XftColors [C_MAIN_FG] = disp->alloc_xftcolor (1.00f, 1.00f, 1.00f, 1.0f);
    XftColors [C_MAIN_L1] = disp->alloc_xftcolor (0.35f, 0.35f, 0.35f, 1.0f);
    XftColors [C_MAIN_L2] = disp->alloc_xftcolor (0.50f, 0.50f, 0.50f, 1.0f);
    XftColors [C_TEXT_BG] = disp->alloc_xftcolor (0.10f, 0.10f, 0.10f, 1.0f);
    XftColors [C_TEXT_FG] = disp->alloc_xftcolor (1.00f, 1.00f, 1.00f, 1.0f);

    XftFonts [F_TEXT1] = disp->alloc_xftfont (xrm->get (".font.text1", "luxi:pixelsize=11"));
    XftFonts [F_TEXT2] = disp->alloc_xftfont (xrm->get (".font.text2", "luxi:bold::pixelsize=9"));

    tstyle1.font = XftFonts [F_TEXT1];
    tstyle1.color.normal.bgnd = XftColors [C_MAIN_BG]->pixel;
    tstyle1.color.normal.text = XftColors [C_MAIN_FG];

    tstyle2.font = XftFonts [F_TEXT2];
    tstyle2.color.normal.bgnd = XftColors [C_MAIN_BG]->pixel;
    tstyle2.color.normal.text = XftColors [C_MAIN_FG];

    bstyle1.font =  XftFonts [F_TEXT2];
    bstyle1.color.bg[0] = XftColors [C_MAIN_BG]->pixel;
    bstyle1.color.fg[0] = XftColors [C_MAIN_FG];
    bstyle1.type = X_button_style::PLAIN;

    bmidi_img = load_img ("bmidi", disp, XftColors [C_MAIN_BG]);
    bintl_img = load_img ("bintl", disp, XftColors [C_MAIN_BG]);
    bhold_img = load_img ("bhold", disp, XftColors [C_MAIN_BG]);
    filtgain_img = load_img ("filtgain", disp, XftColors [C_MAIN_BG]);
    ledcolor_img = load_img ("ledcolor", disp, XftColors [C_MAIN_BG]);
    oscilctl_img = load_img ("oscilctl", disp, XftColors [C_MAIN_BG]);
    inputctl_img = load_img ("inputctl", disp, XftColors [C_MAIN_BG]);
    excitctl_img = load_img ("excitctl", disp, XftColors [C_MAIN_BG]);
    midichan_img = load_img ("midichan", disp, XftColors [C_MAIN_BG]);
}


void styles_fini (X_display *disp)
{
    free_img (filtgain_img);
    free_img (ledcolor_img);
    free_img (oscilctl_img);
    free_img (inputctl_img);
    free_img (excitctl_img);
    free_img (midichan_img);
}
